<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions">

	

	<xsl:template name="convert_to_local_time">
	<!-- Jude Nugent - 12/04/2019 - adding comments to explain variable definations and code operations -->
		<xsl:param name="v_full"/>

			<xsl:variable name="v_yr" select="substring($v_full,1,4)"/>
			<xsl:variable name="v_mn" select="substring($v_full,6,2)"/>
			<xsl:variable name="v_dy" select="substring($v_full,9,2)"/>
			<xsl:variable name="v_hr" select="substring($v_full,12,2)"/>
			<xsl:variable name="v_mi" select="substring($v_full,15,2)"/>
			<xsl:variable name="v_sc" select="substring($v_full,18,2)"/>
			<xsl:variable name="v_off" select="substring($v_full,20,1)"/>
			<!-- no time zone offset -->
			<xsl:variable name="v_offh" select="substring($v_full,21,2)"/>
			<xsl:variable name="v_offm" select="substring($v_full,24,2)"/>
			<!-- inbound ABB data is in UTC so v_utc will equal v_full
				 v_utcf = v_full as no offset and if there was it would repeat format provided in v_full anyway
				v_date = date for v_full
				v_datet = v_full -->
			<xsl:variable name="v_utc" select="concat($v_yr,'-',$v_mn,'-',$v_dy,'T',$v_hr,':',$v_mi,':',$v_sc)"/>
			<xsl:variable name="v_utcf" select="concat($v_yr,'-',$v_mn,'-',$v_dy,'T',$v_hr,':',$v_mi,':',$v_sc,$v_off,$v_offh,':',$v_offm)"/>
			<xsl:variable name="v_date" select="concat($v_yr,'-',$v_mn,'-',$v_dy)"/>
			<xsl:variable name="v_datet" select="concat($v_yr,'-',$v_mn,'-',$v_dy,'T',$v_hr,':',$v_mi,':',$v_sc)"/>

		<!-- Jude Nugent - 12/04/2019 - v_ldom will equal the last day of the month e.g. 30 Apr 2019: : used to roll month, when 1 hour needs added. Year will not roll because not adding 1 hour in Dec -->
			<xsl:variable name="v_ldom">
				<xsl:call-template name="last-day-of-month">
					<xsl:with-param name="date"><xsl:value-of select="$v_date"/></xsl:with-param>
				</xsl:call-template>
			</xsl:variable>

		<!-- Jude Nugent - 12/04/2019 - year and month are not going to change so only new variable is v_dy_ldom-->
			<xsl:variable name="v_yr_ldom" select="substring($v_ldom,1,4)"/>
			<xsl:variable name="v_mn_ldom" select="substring($v_ldom,6,2)"/>
			<xsl:variable name="v_dy_ldom" select="substring($v_ldom,9,2)"/>

			<xsl:variable name="v_dow">
			<!-- Jude Nugent - 12/04/2019 - v_dow will equal day of week i.e. 1 to 7 -->
				<xsl:call-template name="calculate-day-of-the-week">
					<xsl:with-param name="date"><xsl:value-of select="$v_ldom"/></xsl:with-param>
				</xsl:call-template>
			</xsl:variable>

		<!-- Jude Nugent - 12/04/2019 - converting date variables to numbers -->
			<xsl:variable name="n_yr" select="number($v_yr)"/>
			<xsl:variable name="n_mn" select="number($v_mn)"/>
			<xsl:variable name="n_dy" select="number($v_dy)"/>
			<xsl:variable name="n_hr" select="number($v_hr)"/>
			<xsl:variable name="n_mi" select="number($v_mi)"/>
			<xsl:variable name="n_sc" select="number($v_sc)"/>
			<xsl:variable name="n_offh" select="number($v_offh)"/>
			<xsl:variable name="n_offm" select="number($v_offm)"/>

				<!-- Jude Nugent - 12/04/2019 - write out date in square brackets - remove  -->

															
<!-- JN - 15 Apr 2019 - revised the logic for following checks 
	1. check if hour +1 > 23 then set to 00 and add 1 to v_dy  and then check for add 1 to v month and 1 to v year
	2. check month is Apr to Sep and return UTC plus 1 hour
	3. check month is Mar then day > last sunday and time > 01 
	4. check month is Oct then day <= last sunday and time < 02--> 
	
								<xsl:variable name="ndaycheck" select="$v_dy_ldom - $v_dow - 1"/>
								<xsl:choose>
								<!-- Jude Nugent - 12/04/2019 - figure out daylight savings between last sunday in March @ 01:00  ends 02:00 last Sunday Oct
														- Note in winter we are aligned with UTC so no conversion required - just check for summer-->
														
									<xsl:when test="$n_mn &gt; 3 and $n_mn &lt; 10"> <!-- check month first Apr to Sep and return UTC plus 1 hour-->
										<xsl:choose>
											<!-- check if hour +1 > 23 then set to 00 and add 1 to v_dy  and then check for add 1 to v month--> 
											<xsl:when test="($n_hr + 1) &gt; 23 and ($v_dy_ldom = $v_dy)" >
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn+1, '00'),'-01','T','00:',$v_mi,':',$v_sc)"/>
											</xsl:when>
											<xsl:when test="($n_hr + 1) &gt; 23 " >
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn, '00'),'-',format-number($v_dy+1, '00'),'T','00:',$v_mi,':',$v_sc)"/>
											</xsl:when>
											
											<xsl:otherwise>
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn, '00'),'-',format-number($v_dy, '00'),'T',format-number($n_hr + 1, '00'),':',$v_mi,':',$v_sc)"/>
											</xsl:otherwise>
											
										</xsl:choose>
										
										<!--<xsl:value-of select="concat($v_yr,'-',$v_mn,'-',$v_dy,'T',$n_adjhour,':',$v_mi,':',$v_sc)"/> -->
									</xsl:when>
									
									<!-- check for DLS begins sunday in March @ 01:00 -->
									<xsl:when test="$n_mn = '3' and (($n_hr &gt; 1) or ($n_hr = '1')) and ($n_dy &gt; $ndaycheck)">
										<xsl:choose>
										
											<!-- check if hour +1 > 23 then set to 00 and add 1 to v_dy  and then check for add 1 to v month--> 
											<xsl:when test="($n_hr + 1) &gt; 23 and ($v_dy_ldom = $v_dy)" >
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn+1, '00'),'-01','T','00:',$v_mi,':',$v_sc)"/>
											</xsl:when>
											<xsl:when test="($n_hr + 1) &gt; 23 " >
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn, '00'),'-',format-number($v_dy+1, '00'),'T','00:',$v_mi,':',$v_sc)"/>
											</xsl:when>
											
											<xsl:otherwise>
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn, '00'),'-',format-number($v_dy, '00'),'T',format-number($n_hr + 1, '00'),':',$v_mi,':',$v_sc)"/>
											</xsl:otherwise>
											
										</xsl:choose>
<!--										<xsl:value-of select="concat($v_yr,'-',$v_mn,'-',$v_dy,'T',$n_adjhour,':',$v_mi,':',$v_sc)"/> -->
									</xsl:when>
									
									<!-- check DLS ends 02:00 last Sunday Oct -->
									<xsl:when test="$n_mn = '10' and ($n_hr &lt; 2) and (($n_dy &lt; $ndaycheck+1) or ($n_dy = $ndaycheck+1)) ">
										<xsl:choose>
										
											<!-- check if hour +1 > 23 then set to 00 and add 1 to v_dy  and then check for add 1 to v month--> 
											<xsl:when test="($n_hr + 1) &gt; 23 and ($v_dy_ldom = $v_dy)" >
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn+1, '00'),'-01','T','00:',$v_mi,':',$v_sc)"/>
											</xsl:when>
											<xsl:when test="($n_hr + 1) &gt; 23 " >
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn, '00'),'-',format-number($v_dy+1, '00'),'T','00:',$v_mi,':',$v_sc)"/>
											</xsl:when>
											
											<xsl:otherwise>
												<xsl:value-of select="concat($v_yr,'-',format-number($v_mn, '00'),'-',format-number($v_dy, '00'),'T',format-number($n_hr + 1, '00'),':',$v_mi,':',$v_sc)"/>
											</xsl:otherwise>
											
										</xsl:choose>
<!--										<xsl:value-of select="concat($v_yr,'-',$v_mn,'-',$v_dy,'T',$n_adjhour,':',$v_mi,':',$v_sc)"/> -->
									</xsl:when>
									
									<xsl:otherwise>
										<xsl:value-of select="concat($v_yr,'-',format-number($v_mn, '00'),'-',format-number($v_dy, '00'),'T',$v_hr,':',$v_mi,':',$v_sc)"/>
									</xsl:otherwise>
									
								</xsl:choose>

	</xsl:template>
	
	<xsl:template name="last-day-of-month">
	<!-- Jude Nugent - 12/04/2019 - athis function returns date for the last day of the month e.g. supply 04 Dec 2019 and get 31 Dec 2019 returned -->
		<xsl:param name="date"/>
		<xsl:param name="y" select="substring($date, 1, 4)"/>
		<xsl:param name="m" select="substring($date, 6, 2)"/>
		<xsl:param name="cal" select="'312831303130313130313031'"/>
		<xsl:param name="leap" select="not($y mod 4) and $y mod 100 or not($y mod 400)"/>
		<xsl:param name="month-length" select="substring($cal, 2*($m - 1) + 1, 2) + ($m=2 and $leap)" />
		<xsl:value-of select="concat($y, '-', $m, '-', $month-length)" />
	</xsl:template>
	
	<xsl:template name="calculate-day-of-the-week">
	<!-- Jude Nugent - 12/04/2019 - returns day from 1 to 7, Mon=1 to Sun=7 -->
		<xsl:param name="date-time"/>
		<xsl:param name="date" select="substring-before($date-time,'T')"/>
		<xsl:param name="year" select="substring-before($date,'-')"/>
		<xsl:param name="month" select="substring-before(substring-after($date,'-'),'-')"/>
		<xsl:param name="day" select="substring-after(substring-after($date,'-'),'-')"/>
    
		<xsl:variable name="a" select="floor((14 - $month) div 12)"/>
		<xsl:variable name="y" select="$year - $a"/>
		<xsl:variable name="m" select="$month + 12 * $a - 2"/>
   
		<xsl:value-of select="($day + $y + floor($y div 4) - floor($y div 100) + floor($y div 400) + floor((31 * $m) div 12)) mod 7"/>
   
	</xsl:template>

</xsl:stylesheet>
